<?php

namespace Vinkmag\Compatibility;

class Mig_Unyson_To_Codestar {

    const ARRAY_2D_STYLE     = '2d_array';
    const ARRAY_1D_STYLE     = '1d_array';
    const P_TYPE_CONDITIONAL = 'conditional';

    public function init() {

        $options            = get_option( 'theme_mods_vinkmag', [] );
        $fw_options_parent  = empty( $options['fw_options'] ) ? [] : $options['fw_options'];

        $child_options    = get_option( 'theme_mods_vinkmag-child', [] );
        $fw_options_child = empty( $child_options['fw_options'] ) ? [] : $child_options['fw_options'];

        $fw_options = [];
        if( is_child_theme() ){
            // child-theme is active, child-theme-options will be forwarded as the final options array
            $fw_options = $fw_options_child;
        } else {
            $array_keys = array_keys(array_merge($fw_options_parent, $fw_options_child));
            foreach( $array_keys as $single_key ){
                if( array_key_exists($single_key, $fw_options_parent) ){
                    $fw_options[$single_key] = $fw_options_parent[$single_key];
                }else{
                    $fw_options[$single_key] = $fw_options_child[$single_key];
                }
            }
        }

        $modified   = [];
        $original   = [];
        $conf_array = $this->get_config_array();

        foreach ( $conf_array as $codestar_op_key => $unyson_array ) {

            foreach ( $unyson_array as $ky => $conf ) {

                $o_ky = $ky;
                $from = $conf['from'];
                $to   = $conf['to'];
                $ky   = empty( $conf['k_map'] ) ? $ky : trim( $conf['k_map'] );

                if ( empty( $conf['p_type'] ) ) {

                    if ( isset( $fw_options[$o_ky] ) ) {
                        $modified[$ky]   = $this->fw_convert_mapper( $from, $to, $fw_options[$o_ky], $conf );
                        $original[$o_ky] = $fw_options[$o_ky];

                    } elseif ( isset( $conf['def_val'] ) ) {

                        $modified[$ky]   = $conf['def_val'];
                        $original[$o_ky] = $conf['def_val'];
                    }

                } elseif ( $conf['p_type'] === self::P_TYPE_CONDITIONAL ) {

                    $loc_arr = explode( '|', $conf['o_location'] );

                    $tmp = $fw_options;

                    foreach ( $loc_arr as $item ) {

                        $tmp = isset( $tmp[$item] ) ? $tmp[$item] : [];
                    }

                    if ( !empty( $tmp ) ) {
                        $modified[$ky]   = $this->fw_convert_mapper( $from, $to, $tmp[$o_ky] );
                        $original[$o_ky] = $tmp[$o_ky];

                    } elseif ( isset( $conf['def_val'] ) ) {

                        $modified[$ky]   = $conf['def_val'];
                        $original[$o_ky] = $conf['def_val'];
                    }

                } elseif ( $conf['p_type'] === self::ARRAY_1D_STYLE || $conf['p_type'] === self::ARRAY_2D_STYLE ) {

                    if ( isset( $fw_options[$o_ky] ) ) {
                        $modified[$ky]   = $this->fw_convert_mapper( $from, $to, $fw_options[$o_ky], $conf );
                        $original[$o_ky] = $fw_options[$o_ky];

                    } elseif ( isset( $conf['def_val'] ) ) {

                        $modified[$ky]   = $conf['def_val'];
                        $original[$o_ky] = $conf['def_val'];
                    } else {
                        $modified[$ky]   = [];
                        $original[$o_ky] = [];
                    }

                } elseif ( $conf['p_type'] === 'popup_2d_array' ) {

                    $modified[$ky] = [];

                    if ( !empty( $conf['fields_map'] ) ) {

                        $tmp_mod = [];

                        foreach ( $conf['fields_map'] as $fld_key => $fld_conf ) {

                            $old_ky  = $fld_key;
                            $t_from  = $fld_conf['from'];
                            $t_to    = $fld_conf['to'];
                            $fld_key = empty( $fld_conf['k_map'] ) ? $fld_key : trim( $fld_conf['k_map'] );

                            if ( empty( $fw_options[$o_ky] ) ) {

                                $tmp_mod[$fld_key] = isset( $fld_conf['def_val'] ) ? $fld_conf['def_val'] : '';

                            } else {

                                $tmp_mod[$fld_key] = $this->fw_convert_mapper( $t_from, $t_to, $fw_options[$o_ky][$old_ky], $fld_conf );
                            }

                        }

                        $modified[$ky]   = $tmp_mod;
                        $original[$o_ky] = isset( $fw_options[$o_ky] ) ? $fw_options[$o_ky] : [];
                    }

                }

            }

            // $this->debug($original, $modified);
            $this->persist_codestar_settings( $codestar_op_key, $modified );

            $modified = [];
            $original = [];
        }

        $this->migrate_cpt();
        $this->migrate_taxonomy();

    }

    protected function get_config_array() {

        //codestar_slug_id_prefix => which fields goes here...

        $config_array = [

            // Customizer settings
            'vinkmag_section_theme_settings' => array_merge(
                $this->general_settings_config(),
                $this->style_settings_config(),
                $this->header_settings_config(),
                $this->blocks_settings_config(),
                $this->blog_settings_config(),
                $this->footer_settings_config(),
                $this->amp_settings_config(),
                $this->ads_settings_config(),
                $this->optimization_settings_config()
            ),
        ];

        return $config_array;
    }

    protected function general_settings_config() {

        return [
            'general_text_logo'             => [
                'from'    => 'switch',
                'to'      => 'switcher',
                'def_val' => false,
                'v_map'   => [
                    ''    => '',
                    'no'  => 0,
                    'yes' => 1,
                ],
            ],
            'general_text_logo_title'       => [
                'from'       => 'text',
                'to'         => 'text',
                'def_val'    => 'Blog',
                'p_type'     => self::P_TYPE_CONDITIONAL,
                'o_location' => 'general_text_logo_settings|yes', // Muiltipicker key from Unyson setting. This is not dependent with Codestar Freamwork
            ],
            'general_text_logo_title_color' => [
                'from'       => 'color-picker',
                'to'         => 'color',
                'def_val'    => '',
                'p_type'     => self::P_TYPE_CONDITIONAL,
                'o_location' => 'general_text_logo_settings|yes',
            ],
            'general_main_logo'             => [
                'from' => 'upload',
                'to'   => 'media',
            ],
            'general_light_logo'            => [
                'from' => 'upload',
                'to'   => 'media',
            ],
            'blog_breadcrumb_show'          => [
                'from'    => 'switch',
                'to'      => 'switcher',
                'def_val' => true,
                'v_map'   => [
                    ''    => '',
                    'no'  => 0,
                    'yes' => 1,
                ],
            ],
            'blog_breadcrumb_length'        => [
                'from'    => 'text',
                'to'      => 'text',
                'def_val' => '3',
            ],
            'general_social_links'          => [
                'from'   => 'addable-popup',
                'to'     => 'group',
                'p_type' => self::ARRAY_2D_STYLE,
                'o_map'  => [
                    'title'      => 'title',
                    'icon_class' => 'icon_class',
                    'url'        => 'url',
                ],
            ],
            'blog_reading_pregressbar'      => [
                'from'    => 'switch',
                'to'      => 'switcher',
                'def_val' => false,
                'v_map'   => [
                    ''    => '',
                    'no'  => 0,
                    'yes' => 1,
                ],
            ],
            'blog_reading_progressbar_area' => [
                'from'       => 'select-multiple', //TODO: Need to check with Reza Vai
                'to'         => 'select',
                'def_val'    => 'post',
                'p_type'     => self::P_TYPE_CONDITIONAL,
                'o_location' => 'blog_reading_pregressbar|yes',
            ],
            'general_container_width'       => [
                'from'    => 'switch',
                'to'      => 'switcher',
                'def_val' => false,
                'v_map'   => [
                    ''    => '',
                    'no'  => 0,
                    'yes' => 1,
                ],
            ],
            'container_width_laptop'        => [
                'from'       => 'slider',
                'to'         => 'slider',
                'def_val'    => '960',
                'p_type'     => self::P_TYPE_CONDITIONAL,
                'o_location' => 'general_container_width_settings|yes',
            ],
            'container_width_desktop'       => [
                'from'       => 'slider',
                'to'         => 'slider',
                'def_val'    => '1200',
                'p_type'     => self::P_TYPE_CONDITIONAL,
                'o_location' => 'general_container_width_settings|yes',
            ],

        ];
    }

    protected function style_settings_config() {

        return [
            'body_font'            => [
                'from'    => 'typography-v2',
                'to'      => 'typography',
                'p_type'  => self::ARRAY_1D_STYLE,
                'o_map'   => $this->get_typography_v2_map(),
                'f_map'   => [
                    'unit' => 'px',
                    'type' => 'google',
                ],
                'def_val' => [
                    'family'      => 'Arimo',
                    'font-weight' => '400',
                    'size'        => '14',
                    'unit'        => 'px',
                    'type'        => 'google',
                ],
            ],
            'heading_font'         => [
                'from'    => 'typography-v2',
                'to'      => 'typography',
                'p_type'  => self::ARRAY_1D_STYLE,
                'o_map'   => $this->get_typography_v2_map(),
                'f_map'   => [
                    'unit' => 'px',
                    'type' => 'google',
                ],
                'def_val' => [
                    'family'      => 'Heebo',
                    'font-weight' => '',
                    'size'        => '',
                    'unit'        => 'px',
                    'type'        => 'google',
                ],
            ],
            'body_bg_img'          => [
                'from' => 'upload',
                'to'   => 'media',
            ],
            'style_body_bg'        => [
                'from'    => 'color-picker',
                'to'      => 'color',
                'def_val' => '#f0f1f4',
            ],
            'style_primary'        => [
                'from'    => 'color-picker',
                'to'      => 'color',
                'def_val' => '#d72924',
            ],
            'title_color'          => [
                'from'    => 'color-picker',
                'to'      => 'color',
                'def_val' => '#101010',
            ],
            'style_box_shadow'     => [
                'from'    => 'switch',
                'to'      => 'switcher',
                'def_val' => true,
                'v_map'   => [
                    ''    => '',
                    'no'  => 0,
                    'yes' => 1,
                ],
            ],
            'style_theme_setting'  => [
                'from'    => 'switch',
                'to'      => 'switcher',
                'def_val' => false,
                'v_map'   => [
                    ''    => '',
                    'no'  => 0,
                    'yes' => 1,
                ],
            ],
            'style_primary_dark'   => [
                'from'       => 'color-picker',
                'to'         => 'color',
                'def_val'    => '#000000',
            ],
            'box_bg_color'         => [
                'from'       => 'color-picker',
                'to'         => 'color',
                'def_val'    => '#1f1f1f',
                'p_type'     => self::P_TYPE_CONDITIONAL,
                'o_location' => 'style_theme_setting_options|dark',
            ],
            'body_text_color'      => [
                'from'       => 'color-picker',
                'to'         => 'color',
                'def_val'    => '#1f1f1f',
                'p_type'     => self::P_TYPE_CONDITIONAL,
                'o_location' => 'style_theme_setting_options|dark',
            ],
            'title_text_color'     => [
                'from'       => 'color-picker',
                'to'         => 'color',
                'def_val'    => '#ffffff',
                'p_type'     => self::P_TYPE_CONDITIONAL,
                'o_location' => 'style_theme_setting_options|dark',
            ],
            'style_darklight_mode' => [
                'from'    => 'switch',
                'to'      => 'switcher',
                'def_val' => false,
                'v_map'   => [
                    ''    => '',
                    'no'  => 0,
                    'yes' => 1,
                ],
            ],

        ];
    }

    protected function get_typography_v2_map() {

        return [
            'family'         => 'font-family',
            'font-weight'    => 'font-weight',
            'variation'      => 'font-style',
            'line-height'    => 'line-height',
            'letter-spacing' => 'letter-spacing',
            'size'           => 'font-size',
            'color'          => 'color',
        ];
    }

    protected function header_settings_config() {

        return [
            'header_layout_style' => [
                'from'    => 'image-picker',
                'to'      => 'image_select',
                'v_map'   => [
                    'style1'  => 'style1',
                    'style2'  => 'style2',
                    'style3'  => 'style3',
                    'style4'  => 'style4',
                    'style5'  => 'style5',
                    'style6'  => 'style6',
                    'style7'  => 'style7',
                    'style8'  => 'style8',
                    'style9'  => 'style9',
                    'style10' => 'style10',
                    'style11' => 'style11',
                ],
                'def_val' => 'style1',
            ],
            'header_show_topbar'  => [
                'from'    => 'switch',
                'to'      => 'switcher',
                'def_val' => true,
                'v_map'   => [
                    ''    => '',
                    'no'  => 0,
                    'yes' => 1,
                ],
            ],
            'suscribe_btn_text'   => [
                'from'    => 'text',
                'to'      => 'text',
                'def_val' => 'Subscribe',
            ],
            'suscribe_btn_url'    => [
                'from'    => 'text',
                'to'      => 'text',
                'def_val' => '#',
            ],
            'header_nav_sticky'   => [
                'from'    => 'switch',
                'to'      => 'switcher',
                'v_map'   => [
                    ''    => '',
                    'no'  => 0,
                    'yes' => 1,
                ],
                'def_val' => true,
            ],
            'header_show_banner'  => [
                'from'    => 'switch',
                'to'      => 'switcher',
                'v_map'   => [
                    ''    => '',
                    'no'  => 0,
                    'yes' => 1,
                ],
                'def_val' => true,
            ],
            'header_show_search'  => [
                'from'    => 'switch',
                'to'      => 'switcher',
                'v_map'   => [
                    ''    => '',
                    'no'  => 0,
                    'yes' => 1,
                ],
                'def_val' => true,
            ],
        ];
    }

    protected function blocks_settings_config() {

        return [

            'block_category_template'    => [
                'from'    => 'image-picker',
                'to'      => 'image_select',
                'v_map'   => [
                    'style1' => 'style1',
                    'style2' => 'style2',
                    'style3' => 'style3',
                    'style4' => 'style4',
                    'style5' => 'style5',
                    'style6' => 'style6',
                    'style7' => 'style7',
                    'style8' => 'style8',
                    'style9' => 'style9',

                ],
                'def_val' => '',
            ],
            'featured_categories'        => [
                'from' => 'text',
                'to'   => 'text',
            ],
            'block_featured_post_show'   => [
                'from'    => 'switch',
                'to'      => 'switcher',
                'v_map'   => [
                    ''    => '',
                    'no'  => 0,
                    'yes' => 1,
                ],
                'def_val' => true,
            ],
            'block_featured_post_layout' => [
                'from'       => 'image-picker',
                'to'         => 'image_select',
                'v_map'      => [
                    'style1' => 'style1',
                    'style2' => 'style2',
                ],
                'def_val'    => 'style1',
                'p_type'     => self::P_TYPE_CONDITIONAL,
                'o_location' => 'block_featured_post|yes',
            ],
            'block_sidebar_layout'       => [
                'from'    => 'image-picker',
                'to'      => 'image_select',
                'v_map'   => [
                    'sidebar-right' => 'sidebar-right',
                    'sidebar-none'  => 'sidebar-none',
                ],
                'def_val' => 'sidebar-right',
            ],
            'block_crop_desc_switch'     => [
                'from'    => 'switch',
                'to'      => 'switcher',
                'v_map'   => [
                    ''    => '',
                    'no'  => 0,
                    'yes' => 1,
                ],
                'def_val' => false,
            ],
            'block_crop_desc_limit'      => [
                'from'       => 'text',
                'to'         => 'text',
                'def_val'    => '35',
                'p_type'     => self::P_TYPE_CONDITIONAL,
                'o_location' => 'block_crop_desc_switch|yes',
            ],
        ];
    }

    protected function blog_settings_config() {

        return [
            'blog_title'                        => [
                'from' => 'text',
                'to'   => 'text',
            ],
            'blog_header_image'                 => [
                'from' => 'upload',
                'to'   => 'media',
            ],
            'post_header_layout'                => [
                'from'    => 'image-picker',
                'to'      => 'image_select',
                'v_map'   => [
                    'style1' => 'style1',
                    'style2' => 'style2',
                    'style3' => 'style3',
                    'style4' => 'style4',
                    'style5' => 'style5',
                    'style6' => 'style6',
                    'style7' => 'style7',
                    'style8' => 'style8',

                ],
                'def_val' => 'style1',
            ],
            'post_sidebar_layout'               => [
                'from'    => 'image-picker',
                'to'      => 'image_select',
                'v_map'   => [
                    'sidebar-right' => 'sidebar-right',
                    'sidebar-none'  => 'sidebar-none',
                ],
                'def_val' => 'sidebar-right',
            ],
            'blog_sticky_sidebar'               => [
                'from'    => 'switch',
                'to'      => 'switcher',
                'v_map'   => [
                    ''    => '',
                    'no'  => 0,
                    'yes' => 1,
                ],
                'def_val' => true,
            ],
            'blog_view_count_show'              => [
                'from'    => 'switch',
                'to'      => 'switcher',
                'v_map'   => [
                    ''    => '',
                    'no'  => 0,
                    'yes' => 1,
                ],
                'def_val' => true,
            ],
            'blog_author_show'                  => [
                'from'    => 'switch',
                'to'      => 'switcher',
                'v_map'   => [
                    ''    => '',
                    'no'  => 0,
                    'yes' => 1,
                ],
                'def_val' => true,
            ],
            'blog_author_image_show'            => [
                'from'    => 'switch',
                'to'      => 'switcher',
                'v_map'   => [
                    ''    => '',
                    'no'  => 0,
                    'yes' => 1,
                ],
                'def_val' => true,
            ],
            'blog_social_share_show'            => [
                'from'    => 'switch',
                'to'      => 'switcher',
                'v_map'   => [
                    ''    => '',
                    'no'  => 0,
                    'yes' => 1,
                ],
                'def_val' => true,
            ],
            'blog_read_time_show'               => [
                'from'    => 'switch',
                'to'      => 'switcher',
                'v_map'   => [
                    ''    => '',
                    'no'  => 0,
                    'yes' => 1,
                ],
                'def_val' => true,
            ],
            'blog_date_show'                    => [
                'from'    => 'switch',
                'to'      => 'switcher',
                'v_map'   => [
                    ''    => '',
                    'no'  => 0,
                    'yes' => 1,
                ],
                'def_val' => true,
            ],
            'blog_cat_show'                     => [
                'from'    => 'switch',
                'to'      => 'switcher',
                'v_map'   => [
                    ''    => '',
                    'no'  => 0,
                    'yes' => 1,
                ],
                'def_val' => true,
            ],
            'blog_popular_post_show'            => [
                'from'    => 'switch',
                'to'      => 'switcher',
                'v_map'   => [
                    ''    => '',
                    'no'  => 0,
                    'yes' => 1,
                ],
                'def_val' => false,
            ],
            'blog_single_infinite_scroll'       => [
                'from'    => 'switch',
                'to'      => 'switcher',
                'v_map'   => [
                    ''    => '',
                    'no'  => 0,
                    'yes' => 1,
                ],
                'def_val' => false,
            ],
            'blog_single_infinite_scroll_items' => [
                'from'       => 'text',
                'to'         => 'text',
                'def_val'    => '3',
                'p_type'     => self::P_TYPE_CONDITIONAL,
                'o_location' => 'blog_single_infinite_scroll_option|yes',
            ],
            'blog_pagination_style'                => [
                'from'    => 'image-picker',
                'to'      => 'image_select',
                'v_map'   => [
                    'loadmore' => 'loadmore',
                    'number'   => 'number',
                ],
                'def_val' => 'number',
            ],

        ];
    }

    protected function footer_settings_config() {

        return [
            'footer_layout_style'       => [
                'from'    => 'image-picker',
                'to'      => 'image_select',
                'v_map'   => [
                    'style1' => 'style1',
                    'style2' => 'style2',
                ],
                'def_val' => 'style1',
            ],
            'footer_show_newslatter'    => [
                'from'    => 'switch',
                'to'      => 'switcher',
                'v_map'   => [
                    ''    => '',
                    'no'  => 0,
                    'yes' => 1,
                ],
                'def_val' => false,
            ],
            'footer_newslatter_options' => [
                'from'       => 'popup',
                'to'         => 'accordion',
                'p_type'     => 'popup_2d_array',
                'fields_map' => [
                    'newslatter_title'          => [
                        'from' => 'text',
                        'to'   => 'text',
                    ],
                    'newslatter_text'           => [
                        'from' => 'textarea',
                        'to'   => 'textarea',
                    ],
                    'newslatter_form_shortcode' => [
                        'from' => 'text',
                        'to'   => 'text',
                    ],
                ],
                'p_type'     => self::P_TYPE_CONDITIONAL,
                'o_location' => 'footer_newslatter|yes',
            ],
            'instagram_feed'            => [
                'from'    => 'switch',
                'to'      => 'switcher',
                'v_map'   => [
                    ''    => '',
                    'no'  => 0,
                    'yes' => 1,
                ],
                'def_val' => true,
            ],
            'ins_access_token'          => [
                'from'       => 'text',
                'to'         => 'text',
                'def_val'    => '2367672995.1677ed0.dea7a14501d04cd9982c7a0d23c716dd',
                'p_type'     => self::P_TYPE_CONDITIONAL,
                'o_location' => 'instagram_feed|yes',

            ],
            'ins_userId'                => [
                'from'       => 'text',
                'to'         => 'text',
                'def_val'    => '2367672995',
                'p_type'     => self::P_TYPE_CONDITIONAL,
                'o_location' => 'instagram_feed|yes',

            ],
            'ins_limit'                 => [
                'from'       => 'text',
                'to'         => 'text',
                'def_val'    => '6',
                'p_type'     => self::P_TYPE_CONDITIONAL,
                'o_location' => 'instagram_feed|yes',

            ],
            'ins_user_link'             => [
                'from'       => 'text',
                'to'         => 'text',
                'def_val'    => 'https://www.instagram.com/xpeeder/',
                'p_type'     => self::P_TYPE_CONDITIONAL,
                'o_location' => 'instagram_feed|yes',

            ],

            'footer_copyright'          => [
                'from'    => 'text',
                'to'      => 'text',
                'def_val' => '&copy; 2022, Vinkmag. All rights reserved',
            ],
        ];
    }

    protected function amp_settings_config() {
        return [
            'amp_header_logo'               => [
                'from' => 'upload',
                'to'   => 'media',
            ],
            'amp_header_menu'               => [ //need to check with reza vai
                'from' => 'select',
                'to'   => 'select',
            ],
            'amp_header_top_info_show'      => [
                'from'    => 'switch',
                'to'      => 'switcher',
                'v_map'   => [
                    ''    => '',
                    'no'  => 0,
                    'yes' => 1,
                ],
                'def_val' => false,
            ],
            'amp_header_nav_search_section' => [
                'from'    => 'switch',
                'to'      => 'switcher',
                'v_map'   => [
                    ''    => '',
                    'no'  => 0,
                    'yes' => 1,
                ],
                'def_val' => true,
            ],
            'amp_footer_logo'               => [
                'from' => 'upload',
                'to'   => 'media',
            ],
            'amp_footer_newsletter_section' => [
                'from'    => 'switch',
                'to'      => 'switcher',
                'v_map'   => [
                    ''    => '',
                    'no'  => 0,
                    'yes' => 1,
                ],
                'def_val' => true,
            ],
            'amp_footer_mailchimp'          => [
                'from' => 'text',
                'to'   => 'text',
            ],
            'amp_footer_menu'               => [ //need to check with reza vai
                'from' => 'select',
                'to'   => 'select',
            ],
            'amp_footer_copyright'          => [
                'from'    => 'textarea',
                'to'      => 'textarea',
                'def_val' => '&copy; 2022, Vinkmag. All rights reserved',
            ],
            'amp_back_to_top'               => [
                'from'    => 'switch',
                'to'      => 'switcher',
                'v_map'   => [
                    ''    => '',
                    'no'  => 0,
                    'yes' => 1,
                ],
                'def_val' => true,
            ],
        ];
    }

    protected function ads_settings_config() {
        return [
            'top_banner'           => [
                'from'       => 'popup',
                'to'         => 'accordion',
                'p_type'     => 'popup_2d_array',
                'fields_map' => [
                    'ad_link'  => [
                        'from' => 'text',
                        'to'   => 'text',
                    ],
                    'ad_image' => [
                        'from' => 'upload',
                        'to'   => 'media',
                    ],
                    'ad_html'  => [
                        'from' => 'textarea',
                        'to'   => 'textarea',
                    ],
                ],
            ],
            'category_post_banner' => [
                'from'       => 'popup',
                'to'         => 'accordion',
                'p_type'     => 'popup_2d_array',
                'fields_map' => [
                    'ad_link'  => [
                        'from' => 'text',
                        'to'   => 'text',
                    ],
                    'ad_image' => [
                        'from' => 'upload',
                        'to'   => 'media',
                    ],
                    'ad_html'  => [
                        'from' => 'textarea',
                        'to'   => 'textarea',
                    ],
                ],
            ],
            'single_blog_banner'   => [
                'from'       => 'popup',
                'to'         => 'accordion',
                'p_type'     => 'popup_2d_array',
                'fields_map' => [
                    'single_ad_enable'   => [
                        'from'    => 'switch',
                        'to'      => 'switcher',
                        'v_map'   => [
                            ''    => '',
                            'no'  => 0,
                            'yes' => 1,
                        ],
                        'def_val' => false,
                    ],
                    'single_ad_link'     => [
                        'from' => 'text',
                        'to'   => 'text',
                    ],
                    'single_ad_image'    => [
                        'from' => 'upload',
                        'to'   => 'media',
                    ],
                    'single_ad_html'     => [
                        'from' => 'textarea',
                        'to'   => 'textarea',
                    ],
                    'single_ad_position' => [ //need to check with reza vai
                        'from' => 'select',
                        'to'   => 'select',
                    ],
                ],
            ],
        ];
    }

    protected function optimization_settings_config() {
        return [
            'dashicons_off'        => [
                'from'    => 'switch',
                'to'      => 'switcher',
                'v_map'   => [
                    ''    => '',
                    'no'  => 0,
                    'yes' => 1,
                ],
                'def_val' => false,
            ],
            'elementor_icons_off'  => [
                'from'    => 'switch',
                'to'      => 'switcher',
                'v_map'   => [
                    ''    => '',
                    'no'  => 0,
                    'yes' => 1,
                ],
                'def_val' => false,
            ],
            'wp_block_library_off' => [
                'from'    => 'switch',
                'to'      => 'switcher',
                'v_map'   => [
                    ''    => '',
                    'no'  => 0,
                    'yes' => 1,
                ],
                'def_val' => false,
            ],
            'fontawesome_off'      => [
                'from'    => 'switch',
                'to'      => 'switcher',
                'v_map'   => [
                    ''    => '',
                    'no'  => 0,
                    'yes' => 1,
                ],
                'def_val' => false,
            ],
            'wur_css_off'          => [
                'from'    => 'switch',
                'to'      => 'switcher',
                'v_map'   => [
                    ''    => '',
                    'no'  => 0,
                    'yes' => 1,
                ],
                'def_val' => false,
            ],
        ];
    }

    public function fw_convert_mapper( $from, $to, $orig_val, $extra = [] ) {

        if ( $from === 'upload' && $to === 'media' ) {

            return $this->fw_convert_upload_to_media( $orig_val );
        }

        if ( $from === 'image-picker' && $to === 'image_select' ) {

            return $this->fw_convert_img_picker_to_img_select( $orig_val );
        }

        if ( $from === 'switch' && $to === 'switcher' ) {

            return $this->fw_convert_switch_to_switcher( $orig_val, $extra );
        }

        if ( $from === 'switch' && $to === 'button_set' ) {

            return $this->fw_convert_switch_to_switcher( $orig_val, $extra );
        }

        if ( $from === 'text' && $to === 'text' ) {

            return $this->fw_convert_no_change_to_no_change( $orig_val, $extra );
        }

        if ( $from === 'textarea' && $to === 'textarea' ) {

            return $this->fw_convert_no_change_to_no_change( $orig_val, $extra );
        }

        if ( $from === 'select' && $to === 'select' ) {

            return $this->fw_convert_no_change_to_no_change( $orig_val, $extra );
        }

        if ( $from === 'color-picker' && $to === 'color' ) {

            return $this->fw_convert_no_change_to_no_change( $orig_val, $extra );
        }

        if ( $from === 'multi-picker' && $to === 'switcher' ) {

            return $this->fw_convert_mlt_picker_to_switcher( $orig_val, $extra );
        }

        if ( $from === 'multi-picker' && $to === 'image_select' ) {

            return $this->fw_convert_mlt_picker_to_img_select( $orig_val, $extra );
        }

        if ( $from === 'addable-popup' && $to === 'group' ) {

            return $this->fw_convert_addable_popup_to_group( $orig_val, $extra );
        }

        if ( $from === 'typography-v2' && $to === 'typography' ) {

            return $this->fw_convert_typ_v2_to_typography( $orig_val, $extra );
        }

        return $orig_val;
    }

    public function fw_convert_upload_to_media( $input ) {

        if ( empty( $input['attachment_id'] ) ) {
            return;
        }

        $atc = wp_get_attachment_metadata( $input['attachment_id'] );

        return [
            'id'          => $input['attachment_id'],
            'url'         => $input['url'],
            'width'       => $atc['width'],
            'height'      => $atc['height'],
            'thumbnail'   => $input['url'],
            'alt'         => '',
            'title'       => '',
            'description' => '',
        ];
    }

    public function fw_convert_img_picker_to_img_select( $input ) {

        // No need to do anything :D

        return $input;
    }

    public function fw_convert_switch_to_switcher( $input, $conf ) {
        /**
         * Here we are processing 0d array
         */

        if ( empty( $conf['v_map'] ) ) {

            return $input;
        }

        $incoming = is_null( $input ) ? '' : $input;

        //todo need to test it with more values, specially with empty value

        return isset( $conf['v_map'][$incoming] ) ? $conf['v_map'][$incoming] : $input;
    }

    public function fw_convert_no_change_to_no_change( $input, $extra ) {

        // We do not need to do anything :D

        return $input;
    }

    public function fw_convert_mlt_picker_to_switcher( $input, $conf = [] ) {

        $fk = $this->get_multi_picker_key( $conf, $input );

        return $this->convert_multi_picker_array( $fk, $input, $conf );
    }

    private function get_multi_picker_key( $conf, $input ) {

        /**
         * According to ataur bhai the key os the first one's value
         */

        if ( empty( $conf['picker_key'] ) ) {

            return key( $input );
        }

        return $conf['picker_key'];
    }

    private function convert_multi_picker_array( $fk, $input, $conf = [] ) {

        $vl = $input[$fk];

        if ( !empty( $conf['v_map'] ) ) {

            $incoming = is_null( $vl ) ? '' : $vl;

            return isset( $conf['v_map'][$incoming] ) ? $conf['v_map'][$incoming] : $vl;
        }

        if ( $vl === 'yes' ) {

            return 1;
        }

        return '';
    }

    // General settings
    public function fw_convert_mlt_picker_to_img_select( $input, $conf = [] ) {

        return $this->fw_convert_mlt_picker_to_switcher( $input, $conf );
    }

    // Style settings
    public function fw_convert_addable_popup_to_group( $input, $conf = [] ) {

        if ( empty( $input ) ) {

            return [];
        }

        $converted = [];

        foreach ( $input as $item ) {

            if ( empty( $item ) ) {
                $converted[] = [];
            }

            if ( empty( $conf['o_map'] ) ) {

                $converted[] = $item;

            } else {

                $tmp = [];

                foreach ( $conf['o_map'] as $oky => $nky ) {

                    $tmp[$nky] = isset( $item[$oky] ) ? $item[$oky] : '';
                }

                $converted[] = $tmp;
            }

        }

        return $converted;
    }

    // Header settings

    public function fw_convert_typ_v2_to_typography( $input, $conf ) {

        /**
         * Here we are processing 1d array value
         */

        $converted = [];

        if ( !empty( $conf['f_map'] ) ) {

            foreach ( $conf['f_map'] as $fk => $fv ) {

                $converted[$fk] = $fv;
            }

        }

        if ( empty( $input ) ) {

            return $converted;
        }

        if ( !empty( $conf['o_map'] ) ) {

            foreach ( $conf['o_map'] as $ok => $nk ) {
                $converted[$nk] = isset( $input[$ok] ) ? $input[$ok] : '';
            }

        }

        return $converted;
    }

    // Banner settings

    private function persist_codestar_settings( $key, $val ) {

        return update_option( $key, $val );
    }

    // Blog Settings
    protected function migrate_cpt() {

        $cpt = [
            // Page Meta
            'page' => [
                'map' => [
                    'override_default'    => [
                        'from'    => 'switch',
                        'to'      => 'switcher',
                        'def_val' => true,
                        'v_map'   => [
                            ''    => '',
                            'no'  => 0,
                            'yes' => 1,
                        ],
                    ],
                    'header_layout_style' => [
                        'from'    => 'image-picker',
                        'to'      => 'image_select',
                        'v_map'   => [
                            'style1'  => 'style1',
                            'style2'  => 'style2',
                            'style3'  => 'style3',
                            'style4'  => 'style4',
                            'style5'  => 'style5',
                            'style6'  => 'style6',
                            'style7'  => 'style7',
                            'style8'  => 'style8',
                            'style9'  => 'style9',
                            'style10' => 'style10',
                            'style11' => 'style11',
                        ],
                        'def_val' => 'style1',
                    ],
                    'type_header'         => [
                        'from' => 'text',
                        'to'   => 'text',
                    ],
                    'type_footer'         => [
                        'from' => 'text',
                        'to'   => 'text',
                    ],
                ],
            ],
            // Post meta
            'post' => [
                'map' => [
                    'featured_video'      => [
                        'from' => 'text',
                        'to'   => 'text',
                    ],
                    'banner-image'        => [
                        'from' => 'upload',
                        'to'   => 'media',
                    ],
                    'breaking_news'       => [
                        'from'    => 'switch',
                        'to'      => 'switcher',
                        'def_val' => false,
                        'v_map'   => [
                            ''    => '',
                            'no'  => 0,
                            'yes' => 1,
                        ],
                    ],
                    'override_default'    => [
                        'from'    => 'switch',
                        'to'      => 'switcher',
                        'def_val' => false,
                        'v_map'   => [
                            ''    => '',
                            'no'  => 0,
                            'yes' => 1,
                        ],
                    ],
                    'post_header_layout'  => [
                        'from'    => 'image-picker',
                        'to'      => 'image_select',
                        'v_map'   => [
                            'style1' => 'style1',
                            'style2' => 'style2',
                            'style3' => 'style3',
                            'style4' => 'style4',
                            'style5' => 'style5',
                            'style6' => 'style6',
                            'style7' => 'style7',
                            'style8' => 'style8',
                        ],
                        'def_val' => 'style1',
                    ],
                    'post_sidebar_layout' => [
                        'from'    => 'image-picker',
                        'to'      => 'image_select',
                        'v_map'   => [
                            'sidebar-right' => 'sidebar-right',
                            'sidebar-none'  => 'sidebar-none',
                        ],
                        'def_val' => 'sidebar-right',
                    ],
                ],
            ],
        ];

        $log = [];

        foreach ( $cpt as $cpt_name => $cpt_conf ) {

            //$resume_key = 'mig_resume_'.$cpt_name;

            $posts = get_posts(
                [
                    'fields'      => 'ids',
                    'post_type'   => $cpt_name,
                    'post_status' => 'publish',
                    'numberposts' => -1,
                ]
            );

            foreach ( $posts as $pid ) {

                $log[$pid] = $this->convert_meta_key_value( $pid, $cpt_conf );
            }

        }

        // $this->debug($log, []);

    }
    
    // Blog Settings
    protected function migrate_taxonomy() {

        $taxonomy = [
            'category' => [
                'map' => [
                    'featured_upload_img' => [
                        'from'  => 'upload',
                        'to'    => 'media',
                    ],
                    'block_highlight_color' => [
                        'from'      => 'color-picker',
                        'to'        => 'color',
                        'def_val'   => '#d72924',
                    ],
                    'block_secondary_color' => [
                        'from'      => 'color-picker',
                        'to'        => 'color',
                        'def_val'   => '#ffffff',
                    ],
                    'override_default' => [
                        'from'      => 'switch',
                        'to'        => 'switcher',
                        'def_val'   => false,
                        'v_map'     => [
                            ''      => '',
                            'no'    => 0,
                            'yes'   => 1,
                        ],
                    ],
                    'block_style_body_bg' => [
                        'from'      => 'color-picker',
                        'to'        => 'color',
                        'def_val'   => '#f0f1f4',
                    ],
                    'style_box_shadow' => [
                        'from'      => 'switch',
                        'to'        => 'switcher',
                        'def_val'   => true,
                        'v_map'     => [
                            ''      => '',
                            'no'    => 0,
                            'yes'   => 1,
                        ],
                    ],
                    'block_category_template' => [
                        'from' => 'image-picker',
                        'to' => 'image_select',
                        'v_map' => [
                            'style1' => 'style1',
                            'style2' => 'style2',
                            'style3' => 'style3',
                            'style4' => 'style4',
                            'style5' => 'style5',
                            'style6' => 'style6',
                            'style7' => 'style7',
                        ],
                        'def_val' => 'style1',
                    ],
                    'block_sidebar_layout' => [
                        'from' => 'image-picker',
                        'to' => 'image_select',
                        'picker_key' => 'choices',
                        'v_map' => [
                            'sidebar-right' => 'sidebar-right',
                            'sidebar-none' => 'sidebar-none',
                        ],
                        'def_val' => 'sidebar-right',
                    ],

                    'block_featured_post_show' => [
                        'from' => 'switch',
                        'to' => 'switcher',
                        'def_val' => true,
                        'v_map' => [
                            '' => '',
                            'no' => 0,
                            'yes' => 1,
                        ],
                    ],
                    'block_featured_post_layout' => [
                        'from' => 'image-picker',
                        'to' => 'image_select',
                        'v_map' => [
                            'style1' => 'style1',
                            'style2' => 'style2',
                        ],
                        'def_val'    => 'style1',
                        'p_type'     => self::P_TYPE_CONDITIONAL,
                        'o_location' => 'block_featured_post|block_featured_post_show|yes',
                    ],

                    'block_crop_desc_switch' => [
                        'from' => 'switch',
                        'to' => 'switcher',
                        'def_val' => false,
                        'v_map' => [
                            '' => '',
                            'no' => 0,
                            'yes' => 1,
                        ],
                    ],
                    'block_crop_desc_limit' => [
                        'from'          => 'text',
                        'to'            => 'text',
                        'def_val'       => '35',
                        'p_type'        => self::P_TYPE_CONDITIONAL,
                        'o_location'    => 'block_crop_desc|block_crop_desc_switch|yes',
                    ],
                ],
                'mkey'  =>  'vinkmag-category-meta',
            ],
        ];

        $log = [];

        foreach ( $taxonomy as $taxonomy_name => $taxonomy_conf ) {
            $terms = get_terms( 
                [
                    'taxonomy'  => $taxonomy_name,
                    'hide_empty'=> false,
                ]
            );

            foreach ( $terms as $term ) {
                $term_id = $term->term_id;
                $log[$term_id] = $this->convert_tax_meta_key_value( $term_id, $taxonomy_conf['mkey'], $taxonomy_conf );
            }

        }
        // $this->debug($log, []);
    }

    // Footer Settings
    protected function convert_tax_meta_key_value( $term_id, $term_meta_key, $taxonomy_conf ) {

        
        $fw_meta = get_term_meta( $term_id, 'fw_options', true );   // current term meta array saved in wp_termmeta table
        $res     = [];
        $final_meta_value_array = [];

        foreach ( $taxonomy_conf['map'] as $mk => $conf ) {

            if ( !isset( $fw_meta[$mk] ) && !isset( $conf['p_type'] )) {
                continue;
            }

            $o_ky = $mk;            // block_featured_post_layout
            $from = $conf['from'];  // image-picker
            $to   = $conf['to'];    // image_select
            $ky   = empty( $conf['k_map'] ) ? $mk : trim( $conf['k_map'] ); //block_featured_post_layout

            if ( empty( $fw_meta[$mk] ) && empty( $conf['p_type'] ) ) {
                $vl = is_array( $fw_meta[$mk] ) ? [] : '';
                $final_meta_value_array[$ky] = $vl;
                $res['mod'][$ky]  = '';
                $res['orig'][$ky] = '';
                continue;
            }
            

            if ( empty( $conf['p_type'] ) ) {
                $res['mod'][$ky]                = $this->fw_convert_mapper( $from, $to, $fw_meta[$o_ky], $conf );
                $res['orig'][$o_ky]             = $fw_meta[$o_ky];
                $final_meta_value_array[$mk]    = $res['mod'][$ky];

            } elseif ( $conf['p_type'] === self::ARRAY_1D_STYLE || $conf['p_type'] === self::ARRAY_2D_STYLE ) {

                $res['mod'][$ky]                = $this->fw_convert_mapper( $from, $to, $fw_meta[$o_ky], $conf );
                $res['orig'][$o_ky]             = $fw_meta[$o_ky];
                $final_meta_value_array[$ky]    = $res['mod'][$ky];

            } elseif( $conf['p_type'] === self::P_TYPE_CONDITIONAL ){
                $o_val              = isset($fw_meta[$o_ky]) ? $fw_meta[$o_ky] : ''; // ''
                $con_input_array    = $this->fw_handle_conditional_tax( $from, $to, $o_ky, $o_val, $conf, $taxonomy_conf['map'], $fw_meta );
                

                if( is_array($con_input_array) && !empty($con_input_array) ){
                    foreach($con_input_array as $key => $value){
                        $res['mod'][$key]               = $value;
                        $final_meta_value_array[$key]   = $value;
                    }
                }
            }

        }
        
        update_term_meta($term_id, $term_meta_key, $final_meta_value_array);

        return $res;
    }

    // handle conditionals
    protected function fw_handle_conditional_tax( 
        $from, // image-picker
        $to, // image_select
        $unyson_key, // block_featured_post_layout
        $unyson_val = '',       // ''
        $config_array = [],     //'from' => 'image-picker',
        $taxonomy_conf = [],    // the main array containing all key
        $fw_meta = []           // see unserialize.me
        ){

        $array = [];

        if( !isset($config_array['o_location']) ){
            return $array;
        }

        $conditional_hierarchy_arr = explode('|', $config_array['o_location']);
        if( count( $conditional_hierarchy_arr ) < 3 ){
            return $array;
        }
        array_pop($conditional_hierarchy_arr);
        
        
        $root_parent_key = $conditional_hierarchy_arr[0];
        $last_parent_key = $conditional_hierarchy_arr[1];

        if( !isset( $fw_meta[$root_parent_key] ) ){
            return $array;
        }

        $root_parent_vals = $fw_meta[$root_parent_key];

        if( !is_array($root_parent_vals) || empty($root_parent_vals) ){
            return $array;
        }
        
        $last_parent_o_val  = $root_parent_vals[$last_parent_key];  //  yes
        $last_parent_from   = $taxonomy_conf[$last_parent_key]['from'];
        $last_parent_to     = $taxonomy_conf[$last_parent_key]['to'];
        $last_parent_n_val  = $this->fw_convert_mapper( $last_parent_from, $last_parent_to, $last_parent_o_val, $taxonomy_conf[$last_parent_key] ); 
        
        $current_parent_o_val   = !empty($root_parent_vals[$last_parent_o_val][$unyson_key]) ? $root_parent_vals[$last_parent_o_val][$unyson_key] : '';
        $current_parent_n_val   = $this->fw_convert_mapper( $from, $to, $current_parent_o_val, $config_array ); 

        $array[$last_parent_key]    = $last_parent_n_val;
        $array[$unyson_key]         = $current_parent_n_val;

        return $array;
    }

    // Footer Settings
    protected function convert_meta_key_value( $pid, $cpt_conf ) {

        $fw_meta = get_post_meta( $pid, 'fw_options', true );
        $res     = [];

        foreach ( $cpt_conf['map'] as $mk => $conf ) {

            if ( !isset( $fw_meta[$mk] ) ) {
                continue;
            }

            $o_ky = $mk;
            $from = $conf['from'];
            $to   = $conf['to'];
            $ky   = empty( $conf['k_map'] ) ? $mk : trim( $conf['k_map'] );

            if ( empty( $fw_meta[$mk] ) ) {

                $vl = is_array( $fw_meta[$mk] ) ? [] : '';

                update_post_meta( $pid, $ky, $vl );

                $res['mod'][$ky]  = '';
                $res['orig'][$ky] = '';

                continue;
            }

            if ( empty( $conf['p_type'] ) ) {

                $res['mod'][$ky]    = $this->fw_convert_mapper( $from, $to, $fw_meta[$o_ky], $conf );
                $res['orig'][$o_ky] = $fw_meta[$o_ky];

                update_post_meta( $pid, $mk, $res['mod'][$ky] );

            } elseif ( $conf['p_type'] === self::ARRAY_1D_STYLE || $conf['p_type'] === self::ARRAY_2D_STYLE ) {

                $res['mod'][$ky]    = $this->fw_convert_mapper( $from, $to, $fw_meta[$o_ky], $conf );
                $res['orig'][$o_ky] = $fw_meta[$o_ky];

                update_post_meta( $pid, $ky, $res['mod'][$ky] );
            }

        }

        return $res;
    }

    private function debug( $orig, $mod ) {

        echo '<br/>===========================================================================<br/>';
        echo '<pre>';
        print_r( $orig );
        echo '<br/>---------------------------------------------------------------<br/>';
        print_r( $mod );
        echo '</pre>';
    }

}
