<?php if(!defined('ABSPATH')) {
	die('Direct access forbidden.');
}
/*
 * Customizer Option: Style
 * */
CSF::createSection($prefix, [
	'parent' => 'theme_settings',
	'title'  => esc_html__('Style Settings', 'vinkmag'),
	'fields' => [
		// TODO: Need to check with Atik vai about font-family issue.
		[
			'id'       => 'body_font',
			'type'     => 'typography',
			'title'    => esc_html__('Body Font', 'vinkmag'),
			'subtitle' => esc_html__('Choose the typography for the body.', 'vinkmag'),
			'output'   => 'body',

			'default'        => array(
				'font-family' => 'Arimo',
				'font-weight' => '400',
				'font-size'   => '14',
				'unit'        => 'px',
				'type'        => 'google',
			),
			'font_style'     => true,
			'text_align'     => false,
			'line_height'    => false,
			'letter_spacing' => false,
			'text_transform' => false,
			'color'          => false,
			'subset'         => false,
			'preview'		 => 'always'
		],
		[
			'id'       => 'heading_font',
			'type'     => 'typography',
			'title'    => esc_html__('Heading Fonts', 'vinkmag'),
			'subtitle' => esc_html__('Select heading font and styles', 'vinkmag'),
			'output'   => ['h1', 'h2', 'h3', 'h4', 'h5', 'h6'],
			'extra_styles' => true,
			'default'        => array(
				'font-family' => 'Heebo',
				'font-weight' => '500',
				'font-size'   => '',
				'unit'        => 'px',
				'type'        => 'google',
				'extra-styles' => ['400', '500', '600', '700', '800']
			),
			'font_style'     => true,
			'text_align'     => false,
			'line_height'    => false,
			'letter_spacing' => false,
			'text_transform' => false,
			'color'          => false,
			'subset'         => false,
			'preview'		 => 'always'
		],
		[
			'id'             => 'body_bg_img',
			'type'           => 'media',
			'title'          => esc_html__('Body Background Image', 'vinkmag'),
			'subtitle'       => esc_html__('It will be shown on any "dark or coloreful" type area.', 'vinkmag'),
			'url'            => false,
			'preview_width'  => 50,
			'preview_height' => 50,
		],
		[
			'id'       => 'style_body_bg',
			'type'     => 'color',
			'title'    => esc_html__('Body Background Color', 'vinkmag'),
			'subtitle' => esc_html__('Site\'s main background color.', 'vinkmag'),
			'default'  => '#f0f1f4',
		],
		[
			'id'       => 'style_primary',
			'type'     => 'color',
			'title'    => esc_html__('Primary Color', 'vinkmag'),
			'subtitle' => esc_html__('Site\'s main color.', 'vinkmag'),
			'default'  => '#d72924',
		],
		[
			'id'       => 'title_color',
			'type'     => 'color',
			'title'    => esc_html__('Title Color', 'vinkmag'),
			'subtitle' => esc_html__('Site\'s main title color.', 'vinkmag'),
			'default'  => '#101010',
		],
		[
			'id'       => 'style_box_shadow',
			'type'     => 'switcher',
			'title'    => esc_html__('Box Shadow?', 'vinkmag'),
			'subtitle' => esc_html__('Do you want to use box shadow on post blocks?', 'vinkmag'),
			'default'  => true,
			'text_on'  => esc_html__('Yes', 'vinkmag'),
			'text_off' => esc_html__('No', 'vinkmag'),
		],
		[
			'id'       => 'style_theme_setting',
			'type'     => 'switcher',
			'title'    => esc_html__('Theme dark style?', 'vinkmag'),
			'subtitle' => esc_html__('Do you want to use theme style?', 'vinkmag'),
			'default'  => false,
			'text_on'  => esc_html__('Dark', 'vinkmag'),
			'text_off' => esc_html__('Light', 'vinkmag'),
		],
		[
			'id'         => 'style_primary_dark',
			'type'       => 'color',
			'title'      => esc_html__('Primary Dark Body Background', 'vinkmag'),
			'subtitle'   => esc_html__('Site\'s main background color.', 'vinkmag'),
			'default'    => '#000000',
			'dependency' => array('style_theme_setting', '==', 'true')
		],
		[
			'id'         => 'box_bg_color',
			'type'       => 'color',
			'title'      => esc_html__('Content Box Background color', 'vinkmag'),
			'subtitle'   => esc_html__('Dark content box background color.', 'vinkmag'),
			'default'    => '#1f1f1f',
			'dependency' => array('style_theme_setting', '==', 'true')
		],
		[
			'id'         => 'body_text_color',
			'type'       => 'color',
			'title'      => esc_html__('Body Text Color', 'vinkmag'),
			'subtitle'   => esc_html__('Dark body text color.', 'vinkmag'),
			'default'    => '#1f1f1f',
			'dependency' => array('style_theme_setting', '==', 'true')
		],
		[
			'id'         => 'title_text_color',
			'type'       => 'color',
			'title'      => esc_html__('Title Text Color', 'vinkmag'),
			'subtitle'   => esc_html__('Dark title color.', 'vinkmag'),
			'default'    => '#ffffff',
			'dependency' => array('style_theme_setting', '==', 'true')
		],
		[
			'id'       => 'style_darklight_mode',
			'type'     => 'switcher',
			'title'    => esc_html__('Show Dark/Light Icon?', 'vinkmag'),
			'subtitle' => esc_html__('Do you want to show dark/light icon?', 'vinkmag'),
			'default'  => false,
			'text_on'  => esc_html__('Yes', 'vinkmag'),
			'text_off' => esc_html__('No', 'vinkmag'),
		],
	],
]);