<?php if (!defined('ABSPATH')) die('Direct access forbidden.');

/*
 * Vinkmag post category meta.
 * */
CSF::createTaxonomyOptions('vinkmag-category-meta', [
    'taxonomy' => 'category'
]);

// Vinkmag post category meta
CSF::createSection('vinkmag-category-meta', array(
    'fields' => [
        [
            'title'          => esc_html__('Upload Feature Image', 'vinkmag'),
            'desc'           => esc_html__('This will be used as the category feature image', 'vinkmag'),
            'id'             => 'featured_upload_img',
            'type'           => 'media',
            'url'            => false,
            'preview_width'  => 50,
            'preview_height' => 50,
        ],
        [
            'id'      => 'block_highlight_color',
            'type'    => 'color',
            'title'   => esc_html__('Highlight Color', 'vinkmag'),
            'desc'    => esc_html__('This will be used as the primary color for this category', 'vinkmag'),
            'default' => '#d72924',
        ],
        [
            'id'      => 'block_secondary_color',
            'type'    => 'color',
            'title'   => esc_html__('Text Color', 'vinkmag'),
            'desc'    => esc_html__('Choose a color that is easily readable within the highlight color background.', 'vinkmag'),
            'default' => '#ffffff',
        ],
        [
            'id'       => 'override_default',
            'type'     => 'switcher',
            'title'    => esc_html__('Override Default Layouts?', 'vinkmag'),
            'desc'     => esc_html__('You can change the default options from customizer\'s "Blog settings".', 'vinkmag'),
            'default'  => false,
            'text_on'  => esc_html__('Yes', 'vinkmag'),
            'text_off' => esc_html__('No', 'vinkmag'),
        ],
        [
            'id'      => 'block_style_body_bg',
            'type'    => 'color',
            'title'   => esc_html__('Body Background', 'vinkmag'),
            'desc'    => esc_html__('Site\'s main background color.', 'vinkmag'),
            'default' => '#f0f1f4',
        ],
        [
            'id'       => 'style_box_shadow',
            'type'     => 'switcher',
            'title'    => esc_html__('Box Shadow?', 'vinkmag'),
            'desc'     => esc_html__('Do you want to use box shadow on post blocks?', 'vinkmag'),
            'default'  => true,
            'text_on'  => esc_html__('Yes', 'vinkmag'),
            'text_off' => esc_html__('No', 'vinkmag'),
        ],
        [
            'id'      => 'block_category_template',
            'type'    => 'image_select',
            'title'   => esc_html__('Category Template', 'vinkmag'),
            'desc'    => esc_html__('Post block style in category pages.', 'vinkmag'),
            'options' => [
                'style1' => VINKMAG_IMG . '/admin/category-main-layout/min-style2.png',
                'style2' => VINKMAG_IMG . '/admin/category-main-layout/min-style1.png',
                'style3' => VINKMAG_IMG . '/admin/category-main-layout/min-style3.png',
                'style4' => VINKMAG_IMG . '/admin/category-main-layout/min-style4.png',
                'style5' => VINKMAG_IMG . '/admin/category-main-layout/min-style5.png',
                'style6' => VINKMAG_IMG . '/admin/category-main-layout/min-style6.png',
                'style7' => VINKMAG_IMG . '/admin/category-main-layout/min-style9.png',
            ],
            'default' => 'style1'
        ],
        [
            'id'         => 'block_featured_post_show',
            'type'       => 'switcher',
            'title'      => esc_html__('Show Featured Posts', 'vinkmag'),
            'desc'       => esc_html__('Show or hide featured posts in category\'s top section', 'vinkmag'),
            'default'    => true,
            'text_on'    => esc_html__('Yes', 'vinkmag'),
            'text_off'   => esc_html__('No', 'vinkmag'),
            'dependency' => ['block_category_template', '==', 'style7']
        ],
        [
            'id'         => 'block_featured_post_layout',
            'type'       => 'image_select',
            'title'      => esc_html__('Featured Post Style', 'vinkmag'),
            'desc'       => esc_html__('Featured post style in category pages.', 'vinkmag'),
            'options'    => [
                'style1' => VINKMAG_IMG . '/admin/category-header-layout/min-style1.png',
                'style2' => VINKMAG_IMG . '/admin/category-header-layout/min-style2.png',
            ],
            'default'    => 'style1',
            'dependency' => [
                ['block_category_template', '==', 'style7'],
                ['block_featured_post_show', '==', 'true']
            ]
        ],
        [
            'id'      => 'block_sidebar_layout',
            'type'    => 'image_select',
            'title'   => esc_html__('Category Sidebar Layout', 'vinkmag'),
            'desc'    => esc_html__('The sidebar position of a blog category.', 'vinkmag'),
            'options' => [
                'sidebar-right' => VINKMAG_IMG . '/admin/category-sidebar-layout/min-sidebar-right.png',
                'sidebar-none'  => VINKMAG_IMG . '/admin/category-sidebar-layout/min-sidebar-none.png',
            ],
            'default' => 'sidebar-right'
        ],
        [
            'id'       => 'block_crop_desc_switch',
            'type'     => 'switcher',
            'title'    => esc_html__('Custom Post Word Length', 'vinkmag'),
            'desc'     => esc_html__('Enable disable custom post word length.', 'vinkmag'),
            'default'  => false,
            'text_on'  => esc_html__('Yes', 'vinkmag'),
            'text_off' => esc_html__('No', 'vinkmag'),
        ],
        [
            'id'         => 'block_crop_desc_limit',
            'type'       => 'text',
            'title'      => esc_html__('Post Word Limit.', 'vinkmag'),
            'desc'       => esc_html__('Post word limit, example: 100', 'vinkmag'),
            'default'    => '35',
            'dependency' => ['block_crop_desc_switch', '==', 'true'],
        ],
    ]
));
